import ee from "~/event-emitter";

const eventNamespace = "tgam-scroll-manager";

let layers = [];

/**
 * Register a layer
 *   obj: An object with an "isOpen" method
 *   lockEvent: Name of event that will initiate scroll lock (e.g. when layer has finished opening)
 *   unlockEvent: Name of event that will release scroll lock (e.g. when layer has finished closing)
 */
function registerLayer(obj, lockEvent = null, unlockEvent = null) {
  layers.push({
    "object": obj,
    "lockEvent": lockEvent,
    "unlockEvent": unlockEvent
  });
  if (lockEvent) {
    ee.addListener(lockEvent, lockScrolling);
  }
  if (unlockEvent) {
    ee.addListener(unlockEvent, unlockScrolling);
  }
}

function unregisterLayer(obj) {
  layers = layers.filter(layer => {
    if (layer.object === obj) {
      if (layer.lockEvent) {
        ee.removeListener(layer.lockEvent, lockScrolling);
      }
      if (layer.unlockEvent) {
        ee.removeListener(layer.unlockEvent, unlockScrolling);
      }
      return false;
    }
    return true;
  });
}

function lockScrolling() {
  window.document.body.classList.add("u-no-scroll");
  ee.emitEvent(`${eventNamespace}:locked`, [true]);
}

/**
 * When a layer has finished closing, check whether it's safe to allow
 * the body to scroll again (all layers must be closed)
 */
function unlockScrolling() {
  let allLayersClosed = true;
  layers.forEach(layer => {
    if (layer.object.isOpen()) {
      allLayersClosed = false;
    }
  });
  if (allLayersClosed) {
    window.document.body.classList.remove("u-no-scroll");
    ee.emitEvent(`${eventNamespace}:locked`, [false]);
  }
}

export default {
  registerLayer,
  unregisterLayer,
  lockScrolling,
  unlockScrolling
};
